/*
** Copyright (C) 1993-2013 Mark B. Hanson (mbh@panix.com)
**
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2, or (at your option)
** any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

static const char *const file_id =
	"@(#)$Id: fireball.C 1344 2013-04-05 21:18:39Z mark $";

#include "global.h"

#include "random.h"
#include "trig.h"
#include "util.h"
#include "xsc.h"

#include "fireball.h"

using namespace Trig;


namespace {

const coords fireball_points[] = {
    { COS00 / 2.0, -SIN00 / 2.0 }, { -COS00 / 2.0,  SIN00 / 2.0 },
    { COS15 / 2.0, -SIN15 / 2.0 }, { -COS15 / 2.0,  SIN15 / 2.0 },
    { COS30 / 2.0, -SIN30 / 2.0 }, { -COS30 / 2.0,  SIN30 / 2.0 },
    { COS45 / 2.0, -SIN45 / 2.0 }, { -COS45 / 2.0,  SIN45 / 2.0 },
    { COS60 / 2.0, -SIN60 / 2.0 }, { -COS60 / 2.0,  SIN60 / 2.0 },
    { COS75 / 2.0, -SIN75 / 2.0 }, { -COS75 / 2.0,  SIN75 / 2.0 },
    { COS90 / 2.0,  SIN90 / 2.0 }, { -COS90 / 2.0, -SIN90 / 2.0 },
    { COS75 / 2.0,  SIN75 / 2.0 }, { -COS75 / 2.0, -SIN75 / 2.0 },
    { COS60 / 2.0,  SIN60 / 2.0 }, { -COS60 / 2.0, -SIN60 / 2.0 },
    { COS45 / 2.0,  SIN45 / 2.0 }, { -COS45 / 2.0, -SIN45 / 2.0 },
    { COS30 / 2.0,  SIN30 / 2.0 }, { -COS30 / 2.0, -SIN30 / 2.0 },
    { COS15 / 2.0,  SIN15 / 2.0 }, { -COS15 / 2.0, -SIN15 / 2.0 },
};

} // namespace


Fireball::Fireball(void)
{
    //fprintf(stderr, "Fireball::Fireball()\n");
    set_scale(25.0);
    flying = false;
    paused = false;
    points = fireball_points;
    num_points = sizeof(fireball_points) / sizeof(coords);
    xpoints = new XPoint[num_points];
    randomizer = new float[num_points];
    gc = fetch_gc(GC_GREEN);
} // Fireball::Fireball


Fireball::~Fireball(void)
{
    //fprintf(stderr, "Fireball::~Fireball()\n");
    delete[] randomizer;
} // Fireball::~Fireball


void
Fireball::set_randomizer(void)
{
    if (paused) {
	return;
    }

    for (int i = 0; i < num_points; i++) {
	randomizer[i] = (Random::get() % 500) / 1000.0 + 0.50;
    }
} // Fireball::set_randomizer


void
Fireball::render(const bool ink)
{
    if (!alive()) {
	return;
    }

    if (ink) {
	set_randomizer();
	set_xpoints();
    }
    paint_points(ink);
} // Fireball::render


void
Fireball::move(Ship *ship)
{
    if (!alive()) {
	return;
    }

    x += dx;
    y += dy;
    if ((x >= wwidth) || (x < 0) || (y >= gwheight) || (y < 0) ||
	    ship->hit(this)) {
	snuff();
    }
} // Fireball::move


void
Fireball::launch(King *king)
{
    if (alive()) {
	return;
    }

    x = wwidth2;
    y = gwheight2;

    const float deg = king->get_theta();
    const float mag = (wwidth / 2.5) / args.fps;
    dx = xcos(deg) * mag;
    dy = -xsin(deg) * mag;

    flying = true;
} // Fireball::launch


void
Fireball::set_xpoints(void)
{
    for (int i = 0; i < num_points; i++) {
	float nsize = randomizer[i] * size;

	xpoints[i].x = (int)((nsize * points[i].x) + x);
	xpoints[i].y = (int)((nsize * points[i].y) + y);
    }
} // Fireball::set_xpoints
