/*
** Copyright (C) 1993-2013 Mark B. Hanson (mbh@panix.com)
**
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2, or (at your option)
** any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

static const char *const file_id =
	"@(#)$Id: minefield.C 1344 2013-04-05 21:18:39Z mark $";

#include "global.h"

#include "random.h"
#include "util.h"
#include "xsc.h"

#include "minefield.h"


Minefield::Minefield(void)
{
    //fprintf(stderr, "Minefield::Minefield()\n");
    nummines = 3;
    buzzers = new Buzzer[nummines];
} // Minefield::Minefield


Minefield::~Minefield(void)
{
    //fprintf(stderr, "Minefield::~Minefield()\n");
    delete[] buzzers;
} // Minefield::~Minefield


void
Minefield::render(const bool ink) const
{
    for (int i = 0; i < nummines; i++) {
	buzzers[i].render(ink);
    }
} // Minefield::render


void
Minefield::move(Castle *castle, Ship *ship) const
{
    for (int i = 0; i < nummines; i++) {
	buzzers[i].move(castle, ship);
    }
} // Minefield::move


void
Minefield::resize(const int nwidth, const int nheight) const
{
    for (int i = 0; i < nummines; i++) {
	buzzers[i].resize(nwidth, nheight);
    }
} // Minefield::resize


bool
Minefield::hit(Laser *laser) const
{
    for (int i = 0; i < nummines; i++) {
	if (buzzers[i].hit(laser)) {
	    return true;
	}
    }
    return false;
} // Minefield::hit


void
Minefield::launch(King *king, Castle *castle) const
{
    if (!king->alive()) {
	return;
    }

    for (int i = 0; i < nummines; i++) {
	if (!buzzers[i].alive() &&
		(Random::get() % (int)(2 * args.fps) == 0)) {
	    buzzers[i].launch(king, castle);
	    return;
	}
    }
} // Minefield::launch


void
Minefield::upgrade(Castle *castle) const
{
    for (int i = 0; i < nummines; i++) {
	buzzers[i].upgrade(castle);
    }
} // Minefield::upgrade


void
Minefield::snuff(void) const
{
    for (int i = 0; i < nummines; i++) {
	buzzers[i].snuff();
    }
} // Minefield::snuff


void
Minefield::pause(bool pause_state) const
{
    for (int i = 0; i < nummines; i++) {
	buzzers[i].pause(pause_state);
    }
} // Minefield::pause
