/*
** Copyright (C) 1993-2013 Mark B. Hanson (mbh@panix.com)
**
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2, or (at your option)
** any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

static const char *const file_id =
	"@(#)$Id: xything.C 1344 2013-04-05 21:18:39Z mark $";

#include "global.h"

#include "trig.h"
#include "xsc.h"

#include "xything.h"

using namespace Trig;


Xything::Xything(void)
{
    //fprintf(stderr, "Xything::Xything()\n");
    dx = dy = 0.0;
} // Xything::Xything


Xything::~Xything(void)
{
    //fprintf(stderr, "Xything::~Xything()\n");
} // Xything::~Xything


void
Xything::move(void)
{
    x += dx;
    if (x < 0) {
	x += wwidth;
    } else if (x >= wwidth) {
	x -= wwidth;
    }

    y += dy;
    if (y < 0) {
	y += gwheight;
    } else if (y >= gwheight) {
	y -= gwheight;
    }
} // Xything::move


void
Xything::resize(const int nwidth, const int nheight)
{
    dx *= (float)nwidth / wwidth;
    dy *= (float)nheight / gwheight;
    Thing::resize(nwidth, nheight);
} // Xything::resize


void
Xything::bounce(const float phi)
{
    const float cos_phi = xcos(phi);
    const float sin_phi = xsin(phi);

    // see: http://www.psky.com/prog/2dcollision.htm

    // translate the coordinate system so the object can be
    // bounced by merely negating the y portion of its vector
    const float ndx = (dx * cos_phi) + (dy * -sin_phi);
    const float ndy = (dx * sin_phi) + (dy * cos_phi);

    // translate the coordinates back to the original orientation
    // by negating the sin() terms in the above translation
    // (the use of -ndy instead of ndy is the bounce negation mentioned above)
    dx = (ndx * cos_phi) + (-ndy * sin_phi);
    dy = (ndx * -sin_phi) + (-ndy * cos_phi);
} // Xything::bounce
